/*
  Copyright(c) 2009 NEC Corporation. All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  The full GNU General Public License is included in this distribution in the
  file called COPYING.
*/

/* Character-device driver for updating the system bios. */

#ifndef __ONLINEBIOSUPDATE_H__
#define __ONLINEBIOSUPDATE_H__

#include <linux/ioctl.h>
#include <linux/types.h>

#define VERSION "2.0.1"
#define DEVICE_NAME "OnlineBIOSUpdate"

#define TRUE	1
#define FALSE	0
#define STATUS_SUCCESS	0

typedef __u8   BYTE;
typedef __u16  WORD;
typedef __u32  DWORD;
typedef __u64  QWORD;

#define FLAGS_STMICRO_FWH 0x1

#define SPI_SIGNATURE(a, b, c, d)   ((d) << 24 | (c) << 16 | (b) << 8 | (a))
#define SPI_ICH8_SIGNATURE      SPI_SIGNATURE('I','C','H','9')

#pragma pack(1)
typedef struct _SPI_ICH_FLASH_INFO{
//	DWORD		BiosFlashAddress;
//	DWORD		BiosFlashSize;
//	DWORD		FlashAddress;
//	DWORD		DataSize;
	DWORD		Signature;
	DWORD		Size;
	DWORD		SpiFlashWritePeriod;
	DWORD		SpiFlashErasePeriod;
	DWORD		IchPciRcrbAddress;
	DWORD		IchPciRcrbMask;
	DWORD		IchSpiReadStatusControl;
	DWORD		IchSpiReadControl;
	DWORD		IchSpiWriteControl;
	DWORD		IchSpiEraseControl;
	DWORD		IchSpiJedecIdControl;
	WORD		IchSpiBarOffset;
	WORD		IchSpiMappedSize;
	WORD		IchSpiAddressReg;
	WORD		IchSpiDataReg0;
	WORD		IchSpiControlReg;
	WORD		IchSpiStatusReg;
	WORD		IchSpiTypeReg;
	WORD		IchSpiType;
	WORD		IchSpiPrefixReg0;
	WORD		IchSpiCmdReg0;
	BYTE		IchSpiControlRegSize;
	BYTE		IchSpiStatusRegSize;
	BYTE		IchSpiPrefix0;
	BYTE		IchSpiPrefix1;
	BYTE		IchCmd[8];
	BYTE		IchWriteGranularity;
} SPI_ICH_FLASH_INFO;
#pragma pack()

// driver local data structure specific to each device object
typedef struct _LOCAL_DEVICE_INFO{
  unsigned int        FlashBase;
  unsigned char       *pMappedFlashAddress;
  unsigned int        FlashSize;
  unsigned int        Register0;
  unsigned int        FlashMapped;
  unsigned int        SpiUsed;
  BYTE                *pMappedSpiRegs;
  SPI_ICH_FLASH_INFO  IchSpi;
} LOCAL_DEVICE_INFO, *PLOCAL_DEVICE_INFO;

#pragma pack(1)
typedef struct OperationCode {
  unsigned char Cmd;
  unsigned char CmdSize;
  unsigned int  Addr;
  unsigned int  Data;
  unsigned int  Mask;
  unsigned char Pad[1];
  unsigned char Valid;
} UAOpCode;
#pragma pack()

#define OBU_MAGIC 'x'
#define OBU_SET_FLASH_INFO		_IOW(OBU_MAGIC,   1, int[2])
#define OBU_ERASE_FLASH_SECTOR		_IOW(OBU_MAGIC,   2, int)
#define OBU_WRITE_FLASH_SECTOR		_IOW(OBU_MAGIC,   3, int)
#define OBU_ACCESS_RESOURCE		_IOW(OBU_MAGIC,   4, int)
#define OBU_GET_MEMORY_DATA		_IOWR(OBU_MAGIC,  5, int)	// not implemented
#define OBU_SET_SPI_FLASH_INFO		_IOW(OBU_MAGIC,   6, int)
#define OBU_ERASE_SPI_FLASH_SECTOR	_IOW(OBU_MAGIC,   7, int)
#define OBU_WRITE_SPI_FLASH_SECTOR	_IOW(OBU_MAGIC,   8, int)
#define OBU_GET_SPI_FLASH_DATA		_IOWR(OBU_MAGIC,  9, int)	// not implemented
#define OBU_GET_SPI_JEDEC_ID		_IOWR(OBU_MAGIC, 10, int)

// algorism definition
#define RESOURCE_TYPE_IO        0x10
#define RESOURCE_TYPE_MEMORY    0x20
#define RESOURCE_TYPE_PCI       0x30
#define CHECK_RESOURCE_TYPE(a)  (a & 0xf0)
#define COMMAND_TYPE_READ       0x00
#define COMMAND_TYPE_WRITE      0x01
#define COMMAND_TYPE_SET        0x02
#define COMMAND_TYPE_RESET      0x03
#define CHECK_COMMAND_TYPE(a)   (a & 0x0f)
#define COMMAND_SIZE_BYTE       1
#define COMMAND_SIZE_WORD       2
#define COMMAND_SIZE_DWORD      4

#define SPI_COMMAND_WRITE_STATUS    0x01
#define SPI_COMMAND_PROGRAM_BYTE    0x02
#define SPI_COMMAND_READ_DATA       0x03
#define SPI_COMMAND_WRITE_DISABLE   0x04
#define SPI_COMMAND_READ_STATUS     0x05
#define SPI_COMMAND_WRITE_ENABLE    0x06
#define SPI_COMMAND_READ_JEDEC_ID   0x9f

#define SPI_ICH_CONTROL_CMD(Ctrl)              (((Ctrl) & 0x70) >> 4)
#define SPI_ICH_GET_CMD_TYPE(Ctrl, Type)       (((Type) & (3 << (SPI_ICH_CONTROL_CMD(Ctrl) * 2))) >> (SPI_ICH_CONTROL_CMD(Ctrl) * 2))
#define SPI_ICH_SET_DATA_SIZE(Ctrl, Size)      (((Ctrl) & ~SPI_ICH_CTRL_SIZE_MASK) | (((((Size) - 1) << 8)) & SPI_ICH_CTRL_SIZE_MASK))
#define SPI_ICH_CMD_WRITE_MASK          0x1
#define SPI_ICH_CMD_ADDRESS_MASK        0x2
#define SPI_ICH_CTRL_SIZE_MASK          0x3f00
#define SPI_ICH_STATUS_ERROR_MASK       0x8L
#define SPI_ICH_STATUS_DONE_MASK        0x4L
#define SPI_ICH_IN_PROGRESS_MASK        0x1
#define SPI_ICH_MAX_DATA_SIZE       64
#define SPI_FLASH_IN_PROGRESS_MASK      0x1
#define SPI_ICH_NUM_CMD_REGS        8

unsigned int ReadBack;
#define SPI_ICH_READ_REG(a) \
          SPI_ICH_ALIGNED_READ_REG((a), sizeof(*(a)))
#define SPI_ICH_WRITE_REG(a, b) \
          *(a) = b;                \
          ReadBack = SPI_ICH_READ_REG(a);
#define CHECK_WRITE_DATA(a, b) ((SPI_ICH_READ_REG(a) & (b)) == (b) ? TRUE : FALSE)

//
// ICH spi regs should be accessed on natual boundaries
//
BYTE AlignedReadRegByte(BYTE *addr){
  return *(BYTE *)(addr);
}
#define SPI_ICH_ALIGNED_READ_REG(a, size) \
          size == 1 ? AlignedReadRegByte((BYTE *)(a)) :\
            size == 2 ? ((AlignedReadRegByte((BYTE *)(a)+1) << 8) | (AlignedReadRegByte((BYTE *)(a)))) :\
              size == 3 ? ((AlignedReadRegByte((BYTE *)(a)+2) << 16) | (AlignedReadRegByte((BYTE *)(a)+1) << 8) | (AlignedReadRegByte((BYTE *)(a)))) :\
                ((AlignedReadRegByte((BYTE *)(a)+3) << 24) | (AlignedReadRegByte((BYTE *)(a)+2) << 16) | (AlignedReadRegByte((BYTE *)(a)+1) << 8) | (AlignedReadRegByte((BYTE *)(a))))

#define SPI_ICH_ALIGNED_WRITE_REG(a, b, size) \
          SPI_ICH_WRITE_REG(((BYTE *)(a)+(size)-1), (BYTE)((b)>>(8*(size-1)))) \
          ReadBack = SPI_ICH_READ_REG(((BYTE *)(a)+(size)-1)); \
          if(size>=2){ \
            SPI_ICH_WRITE_REG(((BYTE *)(a)+(size)-2), (BYTE)((b)>>(8*(size-2)))) \
            ReadBack = SPI_ICH_READ_REG(((BYTE *)(a)+(size)-2)); \
          } \
          if(size>=3){ \
            SPI_ICH_WRITE_REG(((BYTE *)(a)+(size)-3), (BYTE)((b)>>(8*(size-3)))) \
            ReadBack = SPI_ICH_READ_REG(((BYTE *)(a)+(size)-3)); \
          } \
          if(size==4){ \
            SPI_ICH_WRITE_REG((BYTE *)(a), (BYTE)(b)) \
            ReadBack = SPI_ICH_READ_REG(((BYTE *)(a))); \
          }

/*
 * bios flash driver status
 */
#define OBU_CLOSE      0
#define OBU_OPEN       1

#ifndef KERNEL_VERSION
  #define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
static inline unsigned long msecs_to_jiffies(unsigned long msecs)
{
  return ((HZ * msecs + 999) / 1000);
}
#endif

/*
 * Debugging macro
 */
#ifdef DEBUG
#define OBU_DEBUG(args...) printk(KERN_INFO args);
#else
#define OBU_DEBUG(args...)
#endif

#define GET_BUS_NUMBER(a)        ((a>>16) & 0xff)
#define GET_DEVICE_NUMBER(a)     ((a>>11) & 0x1f)
#define GET_FUNCTION_NUMBER(a)   ((a>>8) & 0x07)
#define GET_REGISTER_NUMBER(a)   (a & 0xff)
#define GET_DEVFUN_NUMBER(a)     ((a>>8) & 0xff)
#endif
